/******** <lalVerbatim file="BlockFinalizeSearchCV"> ********
Author: Tibbits, M. M.
$Id: BlockFinalizeSearch.c,v 1.8 2003/07/06 20:06:08 jolien Exp $
********* </lalVerbatim> ********/

/*************************** <lalLaTeX file="BlockFinalizeSearchC">
\subsection{Module \texttt{BlockFinalizeSearch.c}}
\label{s:BlockFinalizeSearch.c}

Routine that cleans up after the stochastic search.

\subsubsection*{Prototypes}
\input{BlockFinalizeSearchCP}
\index{\texttt{LALFinalizeSearch()}}

\subsection*{Description}

Cleans up after the block normal search--i.e., frees memory previously 
allocated in the \texttt{LALConditionData} routine. 

\subsection*{Algorithms}
  
\subsection*{Uses}

\begin{itemize}
\item \texttt{LALDDestroyVectorSequence} 
\item \texttt{LALDDestroyVector} 
\item \texttt{LALFree}
\end{itemize}

\subsection*{Notes}

\vfill{\footnotesize\input{BlockFinalizeSearchCV}}

*************************************************** </lalLaTeX> */

#include "Block.h"

NRCSID( BLOCKFINALIZESEARCHC, "$Id: BlockFinalizeSearch.c,v 1.8 2003/07/06 20:06:08 jolien Exp $");

/************************ <lalVerbatim file="BlockFinalizeSearchCP"> */
void LALFinalizeSearch
(
	LALStatus	*status,
	void		**searchParams
)
/*********************************************** </lalVerbatim> */
{
	BlockSearchParams	*params;
	BlockResult		*tempResult;
	bandSegment		*tempPointer;
	INT4			iterator;

	INITSTATUS( status, "LALFinalizeSearch", BLOCKFINALIZESEARCHC );
	ATTATCHSTATUSPTR( status );

	params = ((BlockSearchParams*)(*searchParams));

	tempPointer	= NULL;
	tempResult	= NULL;

	if (params->data != NULL)
	{
		  if( params->threshold != NULL )
		    {
		      LALFree( params->threshold );
		    }

		  if( params->meanZero != NULL )
		    {
		      LALFree( params->meanZero );
		    }

		  if( params->varianceZero != NULL )
		    {
		      LALFree( params->varianceZero );
		    }

		  if( params->meanThreshold != NULL )
		    {
		      LALFree( params->meanThreshold );
		    }

		  if( params->meanOffset != NULL )
		    {
		      LALFree( params->meanOffset );
		    }

		  if( params->varianceThreshold != NULL )
		    {
		      LALFree( params->varianceThreshold );
		    }

		  if( params->sampleRate != NULL )
		    {
		      LALFree( params->sampleRate );
		    }

	  if( params->results != NULL )
	    {
	      while ( params->results != NULL )
		{
		  tempResult = params->results;
		  params->results = ((BlockResult*)(params->results->next));
		  LALFree ( tempResult );
		}
	    }

		for(iterator = 0; iterator < params->data->levels; iterator++ )
		{
		  if( params->data->decomposition != NULL )
		    {
		      while( params->data->decomposition[iterator] != NULL)
			{
			  tempPointer = params->data->decomposition[iterator];
			  params->data->decomposition[iterator] = ((bandSegment*)(params->data->decomposition[iterator]->next));

			  LALDDestroyVector( status->statusPtr, &tempPointer->data );
			  CHECKSTATUSPTR( status );
			  LALFree ( tempPointer );
			}
		    }
		}

		if( params->results != NULL )
		  {
		    LALFree( params->results );
		    CHECKSTATUSPTR(status);
		  }

		if( params->data->decomposition != NULL )
		  {
		    LALFree( params->data->decomposition );
		    CHECKSTATUSPTR(status);
		  }

		LALFree(params->data);
		CHECKSTATUSPTR(status);
	}

	if ( params != NULL )
	  {
		LALFree(params);
		CHECKSTATUSPTR(status);
	  }

	DETATCHSTATUSPTR( status );
	RETURN(status);
}
