/******** <lalVerbatim file="FCTInitSearchCV"> ******** 
Author: Wen, L.; Edlund, Jeffrey
$Id: FCTInitSearch.c,v 1.1 2001/12/13 00:32:28 teviet Exp $ 
********* </lalVerbatim> ********/ 


/* These give the number of initialization parameters and the number of parameters 
    per Hierarchy level. 
*/  
/* The initialization parameters come first and are in the following order:
   inputDataLength, numDimensions, numHierLevels, minMass, maxMass, FreqCutoff0

   Then come the parameters per hierarchy level are the following:
   TimeSlice1, FreqCutoff1
   
   Note that the number of hierarchy levels is arbitrary, and the code
   expects TimeSlice and FreqCutOff for each level. For the first
   level the TimeSlice is assumed to be the length of the entire
   dataset.  

*/
/* a general comments:  The range of parameters searched should be constrained  by the number of input datapoint.  Otherwise, it is simply wrong.   The FCT algorithm is now searching millions of parameters for merely two input data points.  -Linqing */

#include <FCTGeneral.h>
#include <FCTInitSearch.h> 

NRCSID( FCTINITSEARCHC, "$Id: FCTInitSearch.c,v 1.1 2001/12/13 00:32:28 teviet Exp $");



void LALInitSearch( 
		   LALStatus           *status, 
		   void               **searchParams, 
		   LALInitSearchParams *initSearchParams 
		   ) 
{ 
  INT4               j,k; /*,dimension,totalNumSeg,dataLength; */  /* ,errorFlag=0 */
  FCTSearchParams   *mySearchParams;
  
  INT4   const       numInitParams = 6;
  INT4   const       numParamsPerHierLevel = 2;
  
  /********************* LAL thing **********************************/ 
  INITSTATUS(status, "LALInitSearch", FCTINITSEARCHC ); /* mandatory for  LAL*/
  ATTATCHSTATUSPTR (status);   

  
#ifdef FCT_SO_ATTACH_DDD
  {
    unsigned int doze = 10;
    pid_t myPID;
    myPID = getpid( );
    
    fprintf(stdout, "pid %d sleeping for %d seconds\n", myPID, doze );
    fflush( stdout );
    
    sleep( doze );   
    fprintf(stdout,"pid %d awake\n", myPID );
    fflush( stdout );
   
  }
#endif
  

  if ( !(initSearchParams) )
  {
    ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL);
  }

  
  /************************************************************ 
		       relevant FCT part 

		       Read in the search parameters
		       initialize FCT   

		       searchMaster= 1: searchmaster; 0:slave; -1: wrappermaster

  **************************************************************/


  /* Allocate memory that doesn't depend on our input. */
  /* allocate the searchParams structure.  Note that the mySearchParams is used 
     in the code because it's type is known. */
  (*searchParams) = mySearchParams = LALMalloc (sizeof( FCTSearchParams ));  
  if ( ! mySearchParams )
  {
    ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
  }
  memset( mySearchParams, 0, sizeof( FCTSearchParams ) );
  
  /* Allocate the fctInParams structure. */ 
  mySearchParams->fctInParams = (FCTInParamType *) LALMalloc (sizeof(FCTInParamType)); 
  if ( !mySearchParams->fctInParams)
  {
    LALFree (searchParams); *searchParams = NULL;
    ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
  }
  memset( mySearchParams->fctInParams, 0, sizeof(FCTInParamType) );

  /* Allocate the data cube structure. */ 
  mySearchParams->fctInParams->cube=(LALFCTDataCube *)LALMalloc(sizeof(LALFCTDataCube));
  /* Note the size of LALFCTDataCube includes the arrays inside it as well. */
  if ( !mySearchParams->fctInParams->cube)
    {
      LALFree (mySearchParams->fctInParams);
      LALFree (searchParams); *searchParams = NULL;
      ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
    }
  memset( mySearchParams->fctInParams->cube, 0, sizeof(LALFCTDataCube) );

  /* Start reading from our arguments. */
  /*  check the validity of the input */
  if ( ! initSearchParams->argv )
  { 
    LALFree (mySearchParams->fctInParams->cube);
    LALFree (mySearchParams->fctInParams);
    LALFree(mySearchParams); mySearchParams = *searchParams = NULL;
    ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL ); 
  }
  
  if ( strcmp( initSearchParams->argv[0], "-filterparams" ) != 0 ) {
    LALFree (mySearchParams->fctInParams->cube);
    LALFree (mySearchParams->fctInParams);
    LALFree(mySearchParams); mySearchParams = *searchParams = NULL;
    ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
  }
  
  /* Now that we've checked that the input is sane, let's start reading it in. */
  /* The input format for the argv is:
     inputDataLength, numDimensions, numHierLevels, minMass, maxMass, FreqCutoff0, TimeSlice1, FreqCutoff1 ... 
  */
  mySearchParams->inputDataLength = atoi(initSearchParams->argv[1]);
  mySearchParams->fctInParams->dataLength=mySearchParams->inputDataLength;
  mySearchParams->fctInParams->dimension = atoi(initSearchParams->argv[2])+1; /* number including dim 0 
									jedlund: Why are we adding 1 to this? We need to pick a standard.*/
  mySearchParams->numHierLevels = atoi(initSearchParams->argv[3]);

  mySearchParams->minMass = atof(initSearchParams->argv[4]);
  mySearchParams->maxMass = atof(initSearchParams->argv[5]);
  
  /* Check to make sure that these inputs are reasonable. */
  if ( (mySearchParams->inputDataLength <= 0) || (mySearchParams->fctInParams->dimension <= 1) || \
       (mySearchParams->fctInParams->dimension > LALFCT_MAX_DIMS) || \
       ( (initSearchParams->argc - numInitParams - 1) != \
	 ((mySearchParams->numHierLevels - 1) * numParamsPerHierLevel))) 
  {
    LALFree (mySearchParams->fctInParams->cube);
    LALFree (mySearchParams->fctInParams);
    LALFree(mySearchParams); mySearchParams = *searchParams = NULL;
    ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
  }
  
  /* Now that we know the number of hierarchy levels, let's allocate arrays that will contain the hierarchy information.*/
  mySearchParams->hierLevelUnits = LALMalloc(sizeof(FCTHierLevelUnitsType) * mySearchParams->numHierLevels );
  mySearchParams->hierLevelEvents = LALMalloc(sizeof(FCTHierEventType *) * ( mySearchParams->numHierLevels + 1 ) );
  mySearchParams->numHierLevelEvents = LALMalloc(sizeof(INT4) * ( mySearchParams->numHierLevels + 1 ) );
  mySearchParams->lowestEvent = LALMalloc(sizeof(INT4) * ( mySearchParams->numHierLevels + 1 ) );
  mySearchParams->powerThreshold = LALMalloc(sizeof(REAL4) * ( mySearchParams->numHierLevels ) );

  if (!mySearchParams->hierLevelUnits || !mySearchParams->hierLevelEvents || !mySearchParams->numHierLevelEvents ||
      !mySearchParams->lowestEvent || !mySearchParams->powerThreshold) {
    if ( mySearchParams->hierLevelUnits )
      LALFree(mySearchParams->hierLevelUnits);
    if (mySearchParams->hierLevelEvents)
      LALFree(mySearchParams->hierLevelEvents);
    if (mySearchParams->numHierLevelEvents)
      LALFree(mySearchParams->numHierLevelEvents);
    if (mySearchParams->lowestEvent)
      LALFree(mySearchParams->lowestEvent);
    if (mySearchParams->powerThreshold)
      LALFree(mySearchParams->powerThreshold);
    LALFree (mySearchParams->fctInParams->cube);
    LALFree (mySearchParams->fctInParams);
    LALFree(mySearchParams); mySearchParams = *searchParams = NULL;
    ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
  }
  memset( mySearchParams->hierLevelUnits, 0, sizeof(FCTHierLevelUnitsType) * mySearchParams->numHierLevels );
  memset( mySearchParams->hierLevelEvents, 0, sizeof(FCTHierEventType *) * ( mySearchParams->numHierLevels + 1 ) );
  memset( mySearchParams->numHierLevelEvents, 0, sizeof(INT4) * ( mySearchParams->numHierLevels + 1 ) );
  memset( mySearchParams->lowestEvent, 0, sizeof(INT4) * ( mySearchParams->numHierLevels + 1 ) );
  memset( mySearchParams->powerThreshold, 0, sizeof(REAL4) * ( mySearchParams->numHierLevels ) );

  /* Each hierarchy level has Units associated with it.  Let's allocate the arrays to store them in. */
  for( j = 0; j < mySearchParams->numHierLevels; j++){
    mySearchParams->hierLevelUnits[j].range = LALMalloc(sizeof(REAL4)*(mySearchParams->fctInParams->dimension));
    mySearchParams->hierLevelUnits[j].physicalStepSize = LALMalloc(sizeof(REAL4)*(mySearchParams->fctInParams->dimension));
    mySearchParams->hierLevelUnits[j].indexRange = LALMalloc(sizeof(INT4)*(mySearchParams->fctInParams->dimension));
    mySearchParams->hierLevelUnits[j].fctUnits=(REAL4 *)LALMalloc(sizeof(REAL4)*mySearchParams->fctInParams->dimension);
    mySearchParams->hierLevelUnits[j].stride=(INT4 *)LALMalloc(sizeof(INT4)*mySearchParams->fctInParams->dimension);

    if (!mySearchParams->hierLevelUnits[j].range || !mySearchParams->hierLevelUnits[j].physicalStepSize \
	|| !mySearchParams->hierLevelUnits[j].indexRange || !mySearchParams->hierLevelUnits[j].fctUnits \
	|| !mySearchParams->hierLevelUnits[j].stride) {
      for (k = 0; k < j; k++) {
	if (mySearchParams->hierLevelUnits[j].range)
	  LALFree(mySearchParams->hierLevelUnits[j].range);
	if (mySearchParams->hierLevelUnits[j].physicalStepSize)
	  LALFree(mySearchParams->hierLevelUnits[j].physicalStepSize);
	if (mySearchParams->hierLevelUnits[j].indexRange)
	  LALFree(mySearchParams->hierLevelUnits[j].indexRange);
	if (mySearchParams->hierLevelUnits[j].fctUnits)
	  LALFree(mySearchParams->hierLevelUnits[j].fctUnits);
	if (mySearchParams->hierLevelUnits[j].stride)
	  LALFree(mySearchParams->hierLevelUnits[j].stride);
      }
      LALFree(mySearchParams->hierLevelUnits);
      LALFree(mySearchParams->hierLevelEvents);
      LALFree(mySearchParams->numHierLevelEvents);
      LALFree(mySearchParams->lowestEvent);
      LALFree(mySearchParams->powerThreshold);
      LALFree (mySearchParams->fctInParams->cube);
      LALFree (mySearchParams->fctInParams);
      LALFree(mySearchParams); mySearchParams = *searchParams = NULL;
      ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS );    
    }
    memset( mySearchParams->hierLevelUnits[j].range, 0, sizeof(REAL4)*(mySearchParams->fctInParams->dimension) );
    memset( mySearchParams->hierLevelUnits[j].physicalStepSize, 0, sizeof(REAL4)*(mySearchParams->fctInParams->dimension) );
    memset( mySearchParams->hierLevelUnits[j].indexRange, 0, sizeof(INT4)*(mySearchParams->fctInParams->dimension) );
    memset( mySearchParams->hierLevelUnits[j].fctUnits, 0, sizeof(REAL4)*mySearchParams->fctInParams->dimension );
    memset( mySearchParams->hierLevelUnits[j].stride, 0, sizeof(INT4)*mySearchParams->fctInParams->dimension );

    if (j == 0) {
      mySearchParams->hierLevelUnits[j].freqCutOff = atof(initSearchParams->argv[numInitParams]);
    } else {
      mySearchParams->hierLevelUnits[j].range[0] = atof(initSearchParams->argv[numInitParams + numParamsPerHierLevel*j - 1]);
      mySearchParams->hierLevelUnits[j].freqCutOff = atof(initSearchParams->argv[numInitParams + numParamsPerHierLevel*j]);
    }    
    if ( ((j != 0) && (mySearchParams->hierLevelUnits[j].range[0] <= 0)) || (mySearchParams->hierLevelUnits[j].freqCutOff <= 0)) {
      for (k = 0; k < j; k++) {
	LALFree(mySearchParams->hierLevelUnits[j].range);
	LALFree(mySearchParams->hierLevelUnits[j].physicalStepSize);
      }
      LALFree(mySearchParams->hierLevelUnits);
      LALFree(mySearchParams->hierLevelEvents);
      LALFree(mySearchParams->numHierLevelEvents);
      LALFree(mySearchParams->lowestEvent);
      LALFree(mySearchParams->powerThreshold);
      LALFree (mySearchParams->fctInParams->cube);
      LALFree (mySearchParams->fctInParams);
      LALFree(mySearchParams); mySearchParams = *searchParams = NULL;
      ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS );    
    }    
  }
  
  /*  Broken debug code. */
  /* #ifdef FCT_SO_DEBUG
     printf("start/end_indices are %i, %i\n",mySearchParams->startIndices[1],mySearchParams->endIndices[1] );
     fprintf (stdout,"totalNumSeg at j=%i are %i\n", j, totalNumSeg);
     fflush( stdout );
     #endif
  */
 
  mySearchParams->curHierLevel = 0;
  mySearchParams->fctInParams->rowsPerApplySearch = 100000;

  /* insertion from Linqing */
  /* setting the threshold */
  mySearchParams->fctInParams->prob=1.0e-7;
  mySearchParams->fctInParams->etaNorm=pow(2*LAL_PI/FCTGENERALH_F0*LAL_PI/25./FCTGENERALH_F0/FCTGENERALH_F0,1./3.); 
  mySearchParams->fctInParams->totalMassNorm=5./32./LAL_PI/LAL_PI/FCTGENERALH_F0/Ts;
  /* allocating memory */
  mySearchParams->fctInParams->freqMoment=(FCTMomentType*)LALMalloc(sizeof(FCTMomentType));
  

#ifdef FCT_SO_DEBUG
  printf("leaving initSearch()\n");
#endif
  
  /******************* LAL thing again ************/  
  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
} 
