/********************************** <lalVerbatim file="PowerApplySearchCV">
Author: Brady, P
$Id: ApplySearch.c,v 1.28 2002/09/20 02:22:10 patrick Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/ExcessPower.h>
#include <lal/TFTransform.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include <lal/BurstSearch.h>
#include <lal/EPData.h>
#include <lal/EPSearch.h>
#include <BuildDB.h>
#include "EPExch.h"
#include "ApplySearch.h"

#define TRUE 1
#define FALSE 0

#define PRINTAVERAGE 1

NRCSID( APPLYSEARCHC, "power $Id: ApplySearch.c,v 1.28 2002/09/20 02:22:10 patrick Exp $");

void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *applyParams
    )
{
  EPSearchParams          *params = NULL;
  INT4                     myRank;
  UINT4                    numEvents=0;
  InitExchParams           initExchParams;
  BurstEvent              *burstEvent=NULL;
  BurstEventList           burstEventList;
  UINT4                    stringLength=0;

  INITSTATUS (status, "LALApplySearch", APPLYSEARCHC);
  ATTATCHSTATUSPTR (status);

  /* check input, output and parameters */
  ASSERT (output, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL);
  ASSERT ( !(output->result), status, APPLYSEARCHH_ENNUL, APPLYSEARCHH_MSGENNUL);
  
  ASSERT (applyParams, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL);
  ASSERT (applyParams->mpiParams, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL);
  ASSERT (applyParams->searchParams, status, APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL);
  
  /* cast the parameter structure, assign MPI information */
  params = (EPSearchParams *) applyParams->searchParams;

  /* 
   *
   * if search master, get the initial number of nodes in the search comm 
   *
   */
  
  if ( params->searchMaster )
  {
    /* only do this the first time the master is called */
    if ( !params->numSlaves )
    {
      /* allocate memory and check for success */
      params->numSlaves = (UINT4 *) LALMalloc( sizeof(UINT4) );
      memset( params->numSlaves, 0, sizeof(UINT4) );
      if( !params->numSlaves )
      {
        ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }

      /* get the number of search slaves */
      MPI_Comm_size( *(applyParams->mpiParams->comm), params->numSlaves );
      *(params->numSlaves) -= 1;
    }
    /* set the number of events to zero */
    numEvents = 0;

    /* initialize the event list */
    {
      stringLength =  APPLYSEARCHH_STRLENGTH;
      burstEventList.ifo = (CHAR *) LALMalloc( stringLength * sizeof(CHAR) );
      strncpy (burstEventList.ifo, params->channelName, stringLength);
    }
  }

  
  /*
   *
   * determine rank and check that we're running on a physical slave
   *
   */

  
  MPI_Comm_rank( *(applyParams->mpiParams->comm), &myRank );

  /* check that the master is running on node rank zero */
  if ( params->searchMaster && myRank != 0 )
  {
    ABORT( status, APPLYSEARCHH_ERANK, APPLYSEARCHH_MSGERANK );
  }

  /* and that the slave is not running on node rank zero */
  if ( !params->searchMaster && myRank == 0 )
  {
    ABORT( status, APPLYSEARCHH_ERANK, APPLYSEARCHH_MSGERANK );
  }
 
  initExchParams.mpiComm   = *(applyParams->mpiParams->comm);
  initExchParams.myProcNum = myRank;

  /* 
   *
   * the main engine starts here.  On the master 
   *
   */

  
  if ( params->searchMaster )
  {
  
    /* if there are slaves */ 
    if ( *(params->numSlaves) )
    {
      ExchParams *thisExch = NULL;

      /* wait for a message */
      LALInitializeExchange (status->statusPtr, &thisExch, NULL, &initExchParams);
      CHECKSTATUSPTR (status);
      
      /* case the exchange type */
      switch (thisExch->exchObjectType)
      {
        
        /* how many segments slave should analyze */
        case ExchDataList:

          /* if there is more data to analyze  */
          if ( params->currentSegment < params->initParams->numSegments )
          {
            UINT4       tmpDutyCycle=0;
            UINT4       dumCurrentSeg=params->currentSegment;

            /* count the segments to analyze */
            for ( tmpDutyCycle=0 ; tmpDutyCycle < params->initParams->segDutyCycle && 
                dumCurrentSeg < params->initParams->numSegments ; tmpDutyCycle++ )
            {
              dumCurrentSeg++;
            }
            
            /* tell the slave how many segments */
            /* these two exchanges should be melded into one */
            LALExchangeUINT4 (status->statusPtr, &tmpDutyCycle, thisExch);
            CHECKSTATUSPTR (status);
            
            /* tell the slave the first segment */
            LALExchangeUINT4 (status->statusPtr, &(params->currentSegment), thisExch);
            CHECKSTATUSPTR (status);

            /* increment to the next segment number to be analyzed */
            params->currentSegment += tmpDutyCycle;
          }
          else
          {
            UINT4       tmpDutyCycle;

            /* tell the slave that there is no more data */
            tmpDutyCycle = 0;
            LALExchangeUINT4 (status->statusPtr, &tmpDutyCycle, thisExch);
            CHECKSTATUSPTR (status);
          }
          break;
  
        /* exchange event list */
        case ExchEPEvent:
          /* get the number of events */
          LALExchangeUINT4 (status->statusPtr, &numEvents, thisExch);
          CHECKSTATUSPTR (status);

          /* if there are events */
          if ( numEvents >0 )
          {
            PowerOutPutParams  outPutParams;

            /* update number of events recorded */
            params->numEvents += numEvents;

            /* set outPutParams */
            outPutParams.numEvents=numEvents;
            outPutParams.flow=params->tfTilingInput->flow;

            /* get the event list and stick them into temporary memory */
            LALExchangeEPEventList(status->statusPtr, &(burstEvent), 
                numEvents, thisExch);
            CHECKSTATUSPTR (status);

            /* point burstEventList to events */
            burstEventList.burstEvent = burstEvent;
            
            /* build the wrapper's output structure */
            BuildPowerOutPut (status->statusPtr, output, &burstEventList, 
                &outPutParams);
            CHECKSTATUSPTR (status);
            
            /* free the temporary memory containin the events */
            while (burstEvent)
            {
              BurstEvent *thisEvent;
              thisEvent = burstEvent;
              burstEvent = burstEvent->nextEvent;
              LALFree( thisEvent );
            }
          }
          /* if there are no events set output to zero */
          else
          {
            output->numOutput     = 0;
            output->result        = NULL;
          }
          break;
      

        /* a slave is finished */
        case ExchFinished:
          --*(params->numSlaves);
          break;


        /* unrecognized type */
        default: 
          ABORT( status, APPLYSEARCHH_EUEXT, APPLYSEARCHH_MSGEUEXT );
      }
      /* end case exhange type */

      /* finish the exchange */
      LALFinalizeExchange (status->statusPtr, &thisExch);
      CHECKSTATUSPTR (status);

      /* set fraction remaining */
      output->fracRemaining = 1.0 - 
        (REAL4)(params->currentSegment - *(params->numSlaves)) / 
        (REAL4)(params->initParams->numSegments);
      /* BEGIN KLUDGE:  do we need this any more? */
      if ( output->fracRemaining < 0.001 ) { output->fracRemaining = 0.01; } 
      /* END KLUDGE */
    }
    else
    {
      SearchSummaryInfo info;
      UINT4             seq;
      int               size;

      MPI_Comm_size( *applyParams->mpiParams->comm, &size );
      info.dsoName = "ldaspower";
      /* get start/end time from "channel" sequence */
      for ( seq = 0; seq < input->numberSequences; ++seq )
      {
        multiDimData *sequence = input->sequences + seq;
        if ( ! strstr( sequence->name, "channel" ) )
          continue;
        info.inputStartTime.gpsSeconds     = sequence->range.dTime.startSec;
        info.inputStartTime.gpsNanoSeconds = sequence->range.dTime.startNan;
        info.inputEndTime.gpsSeconds       = sequence->range.dTime.stopSec;
        info.inputEndTime.gpsNanoSeconds   = sequence->range.dTime.stopNan;
        break;
      }
      info.numEvents = params->numEvents;
      info.numNodes  = size;
      output->fracRemaining       = 0;
      output->notFinished         = 0;
      output->numOutput           = 1;
      output->result              = LALCalloc( 1, sizeof( *output->result ) );
      output->result->search      = burst;
      output->result->significant = 1;
      LALCreateSearchSummaryDataBase( status->statusPtr,
          &output->result->results, &info );
      CHECKSTATUSPTR( status );
    }

    /* free memory allocated for ifo name */
    LALFree(burstEventList.ifo);
  }

  /*
   *
   * if I'm a slave
   *
   */

  else
  {
    UINT4                   tmpDutyCycle;
    ExchParams              exchEvents;
    ExchParams              exchDutyCycle;
    ExchParams              exchFinished;
    ExchParams             *thisExch = NULL;

    /* exchange finished message */
    exchFinished.exchObjectType  = ExchFinished;
    exchFinished.send            = 0;  /* master sends */
    exchFinished.numObjects      = 0;
    exchFinished.partnerProcNum  = 0;  /* master */

    /* exchange duty cycle information */
    exchDutyCycle.exchObjectType = ExchDataList;
    exchDutyCycle.send           = 0; /* master sends */
    exchDutyCycle.numObjects     = 1; /* i send */
    exchDutyCycle.partnerProcNum = 0; /* master */

    /* exchange event information */
    exchEvents.exchObjectType = ExchEPEvent;
    exchEvents.send           = 1; /* i send */
    exchEvents.numObjects     = 1; /* i send */
    exchEvents.partnerProcNum = 0; /* master */


    /* find out if there is more data to analyze */ 
    LALInitializeExchange (status->statusPtr, &thisExch, 
        &exchDutyCycle, &initExchParams);
    CHECKSTATUSPTR (status);

    LALExchangeUINT4 (status->statusPtr, &tmpDutyCycle, thisExch);
    CHECKSTATUSPTR (status);

    /* if there are no segments,  tell the master we're outta here!  */ 
    if ( !tmpDutyCycle )
    {

      LALFinalizeExchange (status->statusPtr, &thisExch);
      CHECKSTATUSPTR (status);

      /* this is the slave shutdown handshake */
      LALInitializeExchange (status->statusPtr, &thisExch, 
          &exchFinished, &initExchParams);
      CHECKSTATUSPTR (status);

      LALFinalizeExchange (status->statusPtr, &thisExch);
      CHECKSTATUSPTR (status);

      /* make sure to tell the wrapperAPI that we're done */
      output->notFinished   = 0;

      DETATCHSTATUSPTR (status);
      RETURN (status);

    }
    else
    {
      /* there are segments,  so find out the first one to analyze */ 
      LALExchangeUINT4 (status->statusPtr, &(params->currentSegment), thisExch);
      CHECKSTATUSPTR (status);
    }

    /* finish the exchange */
    LALFinalizeExchange (status->statusPtr, &thisExch);
    CHECKSTATUSPTR (status);

    /* 
     *
     * This is the main engine of the excess power method
     *
     */

    EPSearch (status->statusPtr, params, &burstEvent, tmpDutyCycle); 
    CHECKSTATUSPTR (status);

    /* Tell master how many events to expect */
    LALInitializeExchange (status->statusPtr, &thisExch, 
        &exchEvents, &initExchParams);
    CHECKSTATUSPTR (status);

    LALExchangeUINT4 (status->statusPtr, &(params->numEvents), thisExch);
    CHECKSTATUSPTR (status);

    if ( params->numEvents > 0 ){
      /* send the master the event list */
      LALExchangeEPEventList(status->statusPtr, &(burstEvent), 
          params->numEvents, thisExch);
      CHECKSTATUSPTR (status);
    }

    LALFinalizeExchange (status->statusPtr, &thisExch);
    CHECKSTATUSPTR (status);

    /* free the temporary memory containin the events */
    while (burstEvent)
    {
      BurstEvent *thisEvent;
      thisEvent = burstEvent;
      burstEvent = burstEvent->nextEvent;
      LALFree( thisEvent );
    }

    /* Dont know if this is right */
    output->numOutput     = 0;
    output->result        = NULL;
    output->fracRemaining = 0;
    output->notFinished   = 1;

  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

