#include <string.h>
#include <lal/LALStdlib.h>
#include <lal/Units.h>
#include <lal/RingSearch.h>
#include <WrapperRingSearch.h>

NRCSID( CONDITIONDATAC, PKG "$Id: ConditionData.c,v 1.1 2002/02/07 22:20:44 jolien Exp $" );

static char ifoNames[][3] = { "H0", "H1", "H2", "L0", "L1", "P0", "P1", "P2" };

void
LALConditionData(
    LALStatus      *status,
    LALSearchInput *inout,
    void           *searchParams,
    LALMPIParams   *mpiParams
    )
{
  RAT4 minusOne = { -1, 0 };
  LALUnitPair unitPair;
  LALUnit     unit;
  RingSearchData          data;
  RingSearchParams       *params;
  REAL4TimeSeries         channel;
  REAL4FrequencySeries    spectrum;
  COMPLEX8FrequencySeries response;
  REAL4Vector             chanvec;
  REAL4Vector             specvec;
  COMPLEX8Vector          respvec;
  UINT4 i;

  INITSTATUS( status, "LALConditionData", CONDITIONDATAC );
  ATTATCHSTATUSPTR( status );

  ASSERT( inout, status,
      WRAPPERRINGSEARCHH_ENULL, WRAPPERRINGSEARCHH_MSGENULL );
  ASSERT( searchParams, status,
      WRAPPERRINGSEARCHH_ENULL, WRAPPERRINGSEARCHH_MSGENULL );

  params = searchParams;

  data.channel  = NULL;
  data.spectrum = NULL;
  data.response = NULL;

  /*
   *
   * Search through input to find data and response sequences and put them
   * into appropriate LAL types.
   *
   */
  for ( i = 0; i < inout->numberSequences; ++i )
  {
    multiDimData *seq = inout->sequences + i;
    if ( strstr( seq->name, "channel" ) ) /* channel data */
    {
      UINT4 name;
      /* extract IFO name */
      for ( name = 0; name < sizeof( ifoNames ) / sizeof( *ifoNames ); ++name )
      {
        if ( strstr( seq->name, ifoNames[name] ) )
        {
          strncpy( params->ifoName, ifoNames[name], sizeof( params->ifoName ) );
          break;
        }
      }
      channel.data = &chanvec;
      strncpy( channel.name, seq->name, sizeof( channel.name ) );
      channel.epoch.gpsSeconds     = seq->range.dTime.startSec;
      channel.epoch.gpsNanoSeconds = seq->range.dTime.startNan;
      channel.deltaT               = seq->range.dTime.timeStepSize;
      channel.f0                   = seq->range.dTime.baseFreq;
      channel.data->length         = seq->range.dTime.numberSamples;
      channel.data->data           = seq->data.real4;
      channel.sampleUnits          = lalADCCountUnit;
      data.channel = &channel;
    }
    else if ( strstr( seq->name, "response" ) ) /* response data */
    {
      response.data = &respvec;
      strncpy( response.name, seq->name, sizeof( response.name ) );
      response.epoch.gpsSeconds     = seq->range.dFreq.gpsStartTimeSec;
      response.epoch.gpsNanoSeconds = seq->range.dFreq.gpsStartTimeNan;
      response.deltaF               = seq->range.dFreq.freqStepSize;
      response.f0                   = seq->range.dFreq.baseFreq;
      response.data->length         = seq->range.dFreq.numberSamples;
      response.data->data           = seq->data.complex8;
      LALUnitRaise( status->statusPtr, &unit, &lalADCCountUnit, &minusOne );
      CHECKSTATUSPTR( status );
      unitPair.unitOne = &lalStrainUnit;
      unitPair.unitTwo = &unit;
      LALUnitMultiply( status->statusPtr, &response.sampleUnits, &unitPair );
      CHECKSTATUSPTR( status );
      data.response = &response;
    }
    else
    {
      /*
      ABORT( status, WRAPPERRINGSEARCHH_ENAME, WRAPPERRINGSEARCHH_MSGENAME );
      */
    }
  }


  if ( ! data.channel )
  {
    ABORT( status, WRAPPERRINGSEARCHH_ECHAN, WRAPPERRINGSEARCHH_MSGECHAN );
  }
  if ( ! data.response )
  {
    ABORT( status, WRAPPERRINGSEARCHH_ERESP, WRAPPERRINGSEARCHH_MSGERESP );
  }

  LALRingSearchConditionData( status->statusPtr, params, &data );
  CHECKSTATUSPTR( status );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}
