#if 0  /* autodoc block */

<lalVerbatim file="TrivialCV">
$Id: Trivial.c,v 1.3 2001/08/27 16:40:45 jolien Exp $
</lalVerbatim>

<lalLaTeX>
\subsection{Module \texttt{Trivial.c}}
\label{ss:Trivial.c}

The LAL-wrapper interface functions for the \texttt{Trivial} shared object.

\subsubsection*{Description}

The routines \verb+LALInitSearch()+, \verb+LALConditionData()+,
\verb+LALApplySearch()+, and \verb+LALFinializeSearch()+ do nothing more than
print the four verses of the song.

\subsubsection*{Algorithm}

\subsubsection*{Uses}

\subsubsection*{Notes}

\vfill{\footnotesize\input{TrivialCV}}

</lalLaTeX>

#endif /* autodoc block */

#include <stdio.h>
#include <lal/LALStdlib.h>
#include <Trivial.h>
#include "TrivialSong.h"

NRCSID( TRIVIALC, "$Id: Trivial.c,v 1.3 2001/08/27 16:40:45 jolien Exp $" ); 

void LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams
    )
{
  TrivialParams *trivialParams;

  INITSTATUS( status, "LALInitSearch", TRIVIALC );

  ASSERT(    searchParams,  status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );
  ASSERT( ! *searchParams,  status, TRIVIALH_ENNUL, TRIVIALH_MSGENNUL );
  ASSERT( initSearchParams, status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );

  *searchParams = trivialParams = LALMalloc( sizeof( *trivialParams ) );
  if ( ! trivialParams )
  {
    ABORT( status, TRIVIALH_EALOC, TRIVIALH_MSGEALOC );
  }

  trivialParams->verse[Mule]   = CHORUS( mule ) MULE;
  trivialParams->verse[Pig]    = CHORUS( pig  ) PIG;
  trivialParams->verse[Fish]   = CHORUS( fish ) FISH;
  trivialParams->verse[Finale] = FINALE;

  if ( initSearchParams->nodeClass ) /* I am the search master */
  {
    /* print an extra poem */
    if ( EOF == puts( PELICAN ) )
    {
      ABORT( status, TRIVIALH_EPUTS, TRIVIALH_MSGEPUTS );
    }
  }

  if ( EOF == puts( trivialParams->verse[Mule] ) )
  {
    ABORT( status, TRIVIALH_EPUTS, TRIVIALH_MSGEPUTS );
  }

  RETURN( status );
}


void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams          *mpiParams
    )
{
  TrivialParams *trivialParams = searchParams;

  INITSTATUS( status, "LALConditionData", TRIVIALC );

  ASSERT( inout,         status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );
  ASSERT( trivialParams, status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );
  mpiParams = NULL;

  if ( EOF == puts( trivialParams->verse[Pig] ) )
  {
    ABORT( status, TRIVIALH_EPUTS, TRIVIALH_MSGEPUTS );
  }

  RETURN( status );
}


void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *params
    )
{
  TrivialParams *trivialParams;

  INITSTATUS( status, "LALApplySearch", TRIVIALC );
  
  ASSERT( output, status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );
  ASSERT( input,  status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );
  ASSERT( params, status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );

  trivialParams = params->searchParams;

  ASSERT( trivialParams, status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );

  output->numOutput     = 0;
  output->result        = NULL;
  output->fracRemaining = 0;
  output->notFinished   = 0;

  if ( EOF == puts( trivialParams->verse[Fish] ) )
  {
    ABORT( status, TRIVIALH_EPUTS, TRIVIALH_MSGEPUTS );
  }

  RETURN( status );
}


void
LALFinalizeSearch(
    LALStatus             *status,
    void                 **searchParams
    )
{
  TrivialParams *trivialParams;

  INITSTATUS( status, "LALFinalizeSearch", TRIVIALC );

  ASSERT(  searchParams, status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );
  ASSERT( *searchParams, status, TRIVIALH_ENULL, TRIVIALH_MSGENULL );

  trivialParams = *searchParams;

  if ( EOF == puts( trivialParams->verse[Finale] ) )
  {
    ABORT( status, TRIVIALH_EPUTS, TRIVIALH_MSGEPUTS );
  }

  LALFree( *searchParams );
  *searchParams = NULL;

  RETURN( status );
}
